@@ -20,7 +20,7 @@ class Stack(Mutable):
         return self._perform_action("redeploy", params=params)
 
     def export(self):
-        if not self._detail_uri:
+        if not self.resource_uri:
             raise ApiError("You must save the object before performing this operation")
-        url = "/".join([self._detail_uri, "export"])
+        url = "/".join([self.resource_uri, "export"])
         return send_request("GET", url, inject_header=False)
